/*
 * Copyright (c) 2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.security;

import java.util.List;

import javax.xml.namespace.QName;

import org.opensaml.core.xml.schema.XSAny;
import org.opensaml.core.xml.schema.impl.XSAnyBuilder;
import org.opensaml.saml.saml2.core.Extensions;
import org.opensaml.saml.saml2.core.impl.ExtensionsBuilder;
import org.springframework.stereotype.Component;

import lombok.NonNull;
import lombok.RequiredArgsConstructor;

@Component
@RequiredArgsConstructor
public class BayernIdSaml2Extension {
	@NonNull
	private final BayernIdProperties properties;

	static final String VERSION_LOCAL_PART = "Version";
	private static final String VERSION_VALUE = "2";
	static final String NAMESPACE_PREFIX = "akdb";
	static final String NAMESPACE_URI = "https://www.akdb.de/request/2018/09";
	static final String CLASSIC_UI_PREFIX = "classic-ui";
	static final String CLASSIC_UI_URI = "https://www.akdb.de/request/2018/09/classic-ui/v1";
	static final String AUTHN_METHODS_LOCAL_NAME = "AuthnMethods";
	static final String AUTHENTICATION_REQUEST_LOCAL_NAME = "AuthenticationRequest";
	static final String ENABLED_LOCAL_NAME = "Enabled";
	static final String REQUESTED_ATTRIBUTES_LOCAL_NAME = "RequestedAttributes";
	private static final String REQUESTED_ATTRIBUTE_LOCAL_NAME = "RequestedAttribute";
	private static final String ATTRIBUTE_NAME = "Name";
	static final String DISPLAY_INFORMATION_LOCAL_NAME = "DisplayInformation";
	static final String ORGANIZATION_DISPLAY_NAME_LOCAL_NAME = "OrganizationDisplayName";

	public Extensions createAkdbExtension() {
		var extensionsElement = new ExtensionsBuilder().buildObject();

		extensionsElement.getUnknownXMLObjects().add(createAuthRequestElement());

		return extensionsElement;
	}

	XSAny createAuthRequestElement() {
		var authRequestElement = new XSAnyBuilder().buildObject(NAMESPACE_URI, AUTHENTICATION_REQUEST_LOCAL_NAME, NAMESPACE_PREFIX);

		authRequestElement.getUnknownAttributes().put(new QName(VERSION_LOCAL_PART), VERSION_VALUE);
		authRequestElement.getUnknownXMLObjects().addAll(List.of(createAuthnMethods(), createRequestedAttributes(), createDisplayInformation()));

		return authRequestElement;
	}

	XSAny createAuthnMethods() {
		var allowedMethodsElement = new XSAnyBuilder().buildObject(NAMESPACE_URI, AUTHN_METHODS_LOCAL_NAME, NAMESPACE_PREFIX);
		var authnMethodMap = properties.getAuthnMethods();

		authnMethodMap.forEach((key, value) -> allowedMethodsElement.getUnknownXMLObjects().add(createMethod(key, value)));

		return allowedMethodsElement;
	}

	XSAny createMethod(String name, String enabledValue) {
		var method = new XSAnyBuilder().buildObject(NAMESPACE_URI, name, NAMESPACE_PREFIX);
		var enabled = new XSAnyBuilder().buildObject(NAMESPACE_URI, ENABLED_LOCAL_NAME, NAMESPACE_PREFIX);
		enabled.setTextContent(enabledValue);

		method.getUnknownXMLObjects().add(enabled);

		return method;
	}

	XSAny createRequestedAttributes() {
		var requestedAttributesElement = new XSAnyBuilder().buildObject(NAMESPACE_URI, REQUESTED_ATTRIBUTES_LOCAL_NAME, NAMESPACE_PREFIX);

		var attributes = properties.getRequestedAttributeUrns();
		List<XSAny> attributeElements = attributes.stream().map(urn -> {
			var element = new XSAnyBuilder().buildObject(NAMESPACE_URI, REQUESTED_ATTRIBUTE_LOCAL_NAME, NAMESPACE_PREFIX);
			element.getUnknownAttributes().put(new QName(ATTRIBUTE_NAME), urn);
			return element;
		}).toList();

		requestedAttributesElement.getUnknownXMLObjects().addAll(attributeElements);

		return requestedAttributesElement;
	}

	XSAny createDisplayInformation() {
		var displayInformationElement = new XSAnyBuilder().buildObject(NAMESPACE_URI, DISPLAY_INFORMATION_LOCAL_NAME, NAMESPACE_PREFIX);
		var version = new XSAnyBuilder().buildObject(CLASSIC_UI_URI, VERSION_LOCAL_PART, CLASSIC_UI_PREFIX);
		var organizationDisplayName = new XSAnyBuilder().buildObject(CLASSIC_UI_URI, ORGANIZATION_DISPLAY_NAME_LOCAL_NAME, CLASSIC_UI_PREFIX);
		organizationDisplayName.setTextContent(properties.getOrganizationDisplayName());

		version.getUnknownXMLObjects().add(organizationDisplayName);
		displayInformationElement.getUnknownXMLObjects().add(version);

		return displayInformationElement;
	}
}
