/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import static de.ozgcloud.antragsraum.attachments.OzgFileTestFactory.NACHRICHT_EVENT_ID;
import static de.ozgcloud.antragsraum.attachments.OzgFileTestFactory.VORGANG_ID;
import static de.ozgcloud.antragsraum.attachments.OzgFileTestFactory.*;
import static de.ozgcloud.antragsraum.nachricht.NachrichtTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.nio.charset.StandardCharsets;
import java.util.Base64;
import java.util.concurrent.CompletableFuture;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.http.HttpStatus;
import org.springframework.http.HttpStatusCode;
import org.springframework.http.MediaType;
import org.springframework.web.multipart.MultipartFile;

import de.ozgcloud.antragsraum.common.InvalidFileTypeException;

@ExtendWith(MockitoExtension.class)
class FileControllerTest {
	@Spy
	@InjectMocks
	private FileController fileController;

	@Mock
	private FileService fileService;

	@Mock
	private FileValidator fileValidator;

	@Nested
	class TestGetFileContent {
		@BeforeEach
		void init() {
			when(fileService.getFile(anyString(), anyString())).thenReturn(OzgFileTestFactory.create());
		}

		@Test
		void shouldCallGetFile() {
			fileController.getFileData(
			  Base64.getEncoder().encodeToString(CHANNEL_ADDRESS.getBytes(StandardCharsets.UTF_8)), FILE_ID);

			verify(fileService).getFile(anyString(), anyString());
		}

		@Test
		void shouldHaveStatusOk() {
			var response = fileController.getFileData(
			  Base64.getEncoder().encodeToString(CHANNEL_ADDRESS.getBytes(StandardCharsets.UTF_8)), FILE_ID);

			assertThat(response.getStatusCode()).isEqualTo(HttpStatusCode.valueOf(HttpStatus.OK.value()));
		}

		@Test
		void shouldHaveContentLength() {
			var response = fileController.getFileData(
			  Base64.getEncoder().encodeToString(CHANNEL_ADDRESS.getBytes(StandardCharsets.UTF_8)), FILE_ID);

			assertThat(response.getHeaders().getContentLength()).isEqualTo(FILE_SIZE);
		}

		@Test
		void shouldHaveContentType() {
			var response = fileController.getFileData(
			  Base64.getEncoder().encodeToString(CHANNEL_ADDRESS.getBytes(StandardCharsets.UTF_8)), FILE_ID);

			assertThat(response.getHeaders().getContentType()).isEqualTo(MediaType.parseMediaType(CONTENT_TYPE));
		}

		@Test
		void shouldHandleNullFile() {
			when(fileService.getFile(anyString(), anyString())).thenReturn(null);

			var response = fileController.getFileData(
			  Base64.getEncoder().encodeToString(CHANNEL_ADDRESS.getBytes(StandardCharsets.UTF_8)), FILE_ID);

			assertThat(response.getStatusCode()).isEqualTo(HttpStatusCode.valueOf(404));
		}

		@Test
		void shouldHaveContentDispositionHeader() {
			var contentDisposition = fileController.getFileData(
				Base64.getEncoder().encodeToString(CHANNEL_ADDRESS.getBytes(StandardCharsets.UTF_8)), FILE_ID)
			  .getHeaders().getContentDisposition();

			assertThat(contentDisposition).hasToString("attachment; filename=\"" + FILE_NAME + "\"");
		}
	}

	@Nested
	class TestGetFile {
		@Test
		void shouldCallService() {
			fileController.getFile(FILE_ID, REPLY_TO_NACHRICHT_ID);

			verify(fileService).getFile(any(), anyString());
		}
	}

	@Nested
	class TestFileUpload {
		private final CompletableFuture<String> fileIdFuture = CompletableFuture.completedFuture(FILE_ID);

		@Mock
		private MultipartFile multipartFile;

		@BeforeEach
		void init() {
			when(fileService.upload(anyString(), anyString(), any())).thenReturn(fileIdFuture);
			when(fileValidator.isValid(any())).thenReturn(true);
		}

		@Test
		void shouldCallService() {
			fileController.uploadFile(VORGANG_ID, NACHRICHT_EVENT_ID, multipartFile);

			verify(fileService).upload(anyString(), anyString(), any(MultipartFile.class));
		}
	}

	@Nested
	class TestFileUploadExceptions {
		@Mock
		private MultipartFile multipartFile;

		@Test
		void shouldThrowExceptionWhenVorgangIdIsNull() {
			assertThatExceptionOfType(IllegalArgumentException.class).isThrownBy(
			  () -> fileController.uploadFile(null, NACHRICHT_EVENT_ID, multipartFile));
		}

		@Test
		void shouldThrowExceptionWhenRueckfrageIdIsNull() {
			assertThatExceptionOfType(IllegalArgumentException.class).isThrownBy(
			  () -> fileController.uploadFile(VORGANG_ID, null, multipartFile));
		}

		@Test
		void shouldThrowExceptionWhenFileIsInvalid() {
			when(fileValidator.isValid(any())).thenReturn(false);

			assertThatExceptionOfType(InvalidFileTypeException.class).isThrownBy(
			  () -> fileController.uploadFile(VORGANG_ID, NACHRICHT_EVENT_ID, multipartFile));
		}
	}
}