/*
 * Copyright (c) 2023-2024.   Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten
 * des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung.
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.security;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.security.authentication.AnonymousAuthenticationToken;
import org.springframework.security.authentication.RememberMeAuthenticationToken;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;

class AuthenticationHelperTest {
	@Nested
	class TestGetAuthentication {
		@Test
		void shouldGetAuthenticatedUser() {
			var authenticated = mock(RememberMeAuthenticationToken.class);
			var context = mock(SecurityContext.class);
			when(context.getAuthentication()).thenReturn(authenticated);
			SecurityContextHolder.setContext(context);

			var authentication = AuthenticationHelper.getAuthentication();

			assertThat(authentication).isNotNull();
		}

		@Test
		void shouldThrowException() {
			var notAuthenticated = mock(AnonymousAuthenticationToken.class);
			var context = mock(SecurityContext.class);
			when(context.getAuthentication()).thenReturn(notAuthenticated);
			SecurityContextHolder.setContext(context);

			assertThatExceptionOfType(IllegalStateException.class).isThrownBy(AuthenticationHelper::getAuthentication);
		}
	}

	@Nested
	class TestGetSamlToken {
		@Test
		void shouldGetAuthenticatedSamlToken() {
			var authenticated = mock(UsernamePasswordAuthenticationToken.class);
			when(authenticated.getPrincipal()).thenReturn(UserTestFactory.create());
			var context = mock(SecurityContext.class);
			when(context.getAuthentication()).thenReturn(authenticated);
			SecurityContextHolder.setContext(context);

			var samlToken = AuthenticationHelper.getSamlToken();

			assertThat(samlToken).isEqualTo(UserTestFactory.SAML_TOKEN);
		}

		@Test
		void shouldGetEmptySamlToken() {
			var authenticated = mock(RememberMeAuthenticationToken.class);
			var context = mock(SecurityContext.class);
			when(context.getAuthentication()).thenReturn(authenticated);
			SecurityContextHolder.setContext(context);

			var samlToken = AuthenticationHelper.getSamlToken();

			assertThat(samlToken).isEqualTo(AuthenticationHelper.NO_SAML_TOKEN);
		}
	}
}