/*
 * Copyright (c) 2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.security;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.io.IOException;
import java.util.Collection;
import java.util.List;

import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.servlet.http.HttpSession;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UserDetailsService;

@ExtendWith(MockitoExtension.class)
class SamlUrlAuthenticationSuccessHandlerTest {
	private static final String HTTP_TEST = "http://test";
	private static final String REDIRECT_URL = "http://redirect";

	private SamlUrlAuthenticationSuccessHandler successHandler;

	@Mock
	private UserDetailsService userService;

	@Nested
	class TestOnSuccess {
		@Mock
		private HttpServletRequest request;
		@Mock
		private HttpServletResponse response;
		@Mock
		private Authentication authentication;

		@Spy
		SamlUrlAuthenticationSuccessHandler handler = new SamlUrlAuthenticationSuccessHandler(REDIRECT_URL, userService);

		@BeforeEach
		void setup() {
			Collection<? extends GrantedAuthority> grantedAuthorities = List.of(new DefaultRole());
			doReturn(grantedAuthorities).when(authentication).getAuthorities();
		}

		@Test
		void shouldCallHandle() throws IOException {
			handler.onAuthenticationSuccess(request, response, authentication);

			verify(handler).handle(any(), any(), any());
		}

		@Test
		void shouldCallClearAuthenticationAttributes() throws IOException {
			handler.onAuthenticationSuccess(request, response, authentication);

			verify(handler).clearAuthenticationAttributes(any());
		}
	}

	@Nested
	class TestInit {
		@Test
		void shouldGetRedirectUrl() {
			successHandler = new SamlUrlAuthenticationSuccessHandler(REDIRECT_URL, userService);

			var url = successHandler.getRedirectUrl(REDIRECT_URL);

			assertThat(url).isEqualTo(REDIRECT_URL);
		}

		@Test
		void shouldGetDefaultRedirectUrl() {
			successHandler = new SamlUrlAuthenticationSuccessHandler("", userService);

			var url = successHandler.getRedirectUrl("");

			assertThat(url).isEqualTo("/");
		}
	}

	@Nested
	class TestWithAuthentication {
		@Mock
		private Authentication authentication;

		@Test
		void shouldDetermineTargetUrlForRole() {
			Collection<? extends GrantedAuthority> grantedAuthorities = List.of(new DefaultRole());
			doReturn(grantedAuthorities).when(authentication).getAuthorities();

			successHandler = new SamlUrlAuthenticationSuccessHandler("", userService);

			var url = successHandler.determineTargetUrl(authentication);

			assertThat(url).isEqualTo("/");
		}

		@Test
		void shouldDetermineTargetUrlMissingRole() {
			Collection<? extends GrantedAuthority> grantedAuthorities = List.of();
			doReturn(grantedAuthorities).when(authentication).getAuthorities();

			successHandler = new SamlUrlAuthenticationSuccessHandler(REDIRECT_URL, userService);

			assertThatExceptionOfType(IllegalStateException.class).isThrownBy(() -> successHandler.determineTargetUrl(authentication))
			  .withMessage("Invalid role! User is missing role ROLE_USER");
		}
	}

	@Nested
	class TestClearLoginSession {
		@Mock
		private HttpServletRequest request;

		@Mock
		private HttpSession session;

		@Test
		void shouldHandleNullSession() {
			successHandler = new SamlUrlAuthenticationSuccessHandler(REDIRECT_URL, userService);
			successHandler.clearAuthenticationAttributes(request);

			verify(request, times(1)).getSession(false);
		}

		@Test
		void shouldClearSession() {
			when(request.getSession(anyBoolean())).thenReturn(session);

			successHandler = new SamlUrlAuthenticationSuccessHandler(REDIRECT_URL, userService);
			successHandler.clearAuthenticationAttributes(request);

			verify(request, times(1)).getSession(false);
			verify(session).removeAttribute(any());
		}
	}

	@Nested
	class TestHandle {
		@Mock
		private HttpServletRequest request;
		@Mock
		private HttpServletResponse response;
		@Mock
		private Authentication authentication;

		@BeforeEach
		void setup() {
			Collection<? extends GrantedAuthority> grantedAuthorities = List.of(new DefaultRole());
			doReturn(grantedAuthorities).when(authentication).getAuthorities();
		}

		@Test
		void shouldHandle() throws IOException {
			when(request.getContextPath()).thenReturn(HTTP_TEST);
			when(response.isCommitted()).thenReturn(Boolean.FALSE);
			when(response.encodeRedirectURL(anyString())).thenReturn(HTTP_TEST + "/?code=");

			successHandler = new SamlUrlAuthenticationSuccessHandler("", userService);
			successHandler.handle(request, response, authentication);

			verify(response).encodeRedirectURL(startsWith(HTTP_TEST + "/?code="));
			verify(response).sendRedirect(startsWith(HTTP_TEST + "/?code="));
		}

		@Test
		void shouldHandleCommitted() throws IOException {
			when(response.isCommitted()).thenReturn(Boolean.TRUE);
			successHandler = new SamlUrlAuthenticationSuccessHandler(REDIRECT_URL, userService);
			successHandler.handle(request, response, authentication);

			verify(response, never()).sendRedirect(anyString());
		}
	}

}